

# vim: syntax=puppet
# Author: Christopher Gutteridge

# This is a wrapper for creating an eprints repo. 

define uos_eprints_ras::repository (
    String $repo_id = $title,               # The repository ID
    String $service_status,                 # dev|pprd|prod
    String $repo_fqdn,                      # The fully qualifiied domain name of this repository
    String $git_repo,                       # The git URL for the repository with the config files
    String $ssh_identity,                   # The path to the ssh identity file to use for git
    String $archive_name,                   # The name of this repository
    String $org,                            # The name of the organisation that runs this repository
    String $archives_dir = "/srv/eprints/archives",  # Where to place the archives (needs to be the "big" disk)
) {
  include uos_eprints_ras
  include uos_local_password

  $mysql_password = uos_local_password::password($repo_id)

  # create all the config files

  $template_values = {
      'repo_id'           => $repo_id,
      'repo_fqdn'         => $repo_fqdn,
      'service_status'    => $service_status,
      'dbname'            => $repo_id,
      'dbuser'            => $repo_id,
      'dbpass'            => $mysql_password,
      'archive_name'      => $archive_name,
      'org'               => $org,
  }
 
  file { "${archives_dir}/${repo_id}":
    ensure  => 'directory',
    notify  => [ Service['httpd'], Exec["eprints_generate_static_${repo_id}"] ],
    seltype => "httpd_sys_content_t",
    owner   => 'eprints',
    group   => 'eprints',
  }

  case $service_status {
    'prod':  { $git_branch = 'Production' }
    'pprd':  { $git_branch = 'Pre-Production' }
    default: { $git_branch = 'master' }
  }

  vcsrepo { "${archives_dir}/${repo_id}/cfg":
    ensure       => present,
    user         => 'eprints',
    group        => 'eprints',
    provider     => git,
    source       => $git_repo,
    revision     => $git_branch,
    require      => [ File['/home/eprints/.ssh/id_rsa'], File["${archives_dir}/${repo_id}"] ],
    notify       => [ Service['httpd'], Exec["eprints_generate_static_${repo_id}"] ],
  }

  # iSolutions NTP friendly SSL apacheconf 
  # and other config files we need to insert parametrs into
  file { 
    default: 
      seltype => "httpd_sys_content_t",
      owner   => 'eprints',
      group   => 'eprints',
      notify  => Service['httpd'],
      require => Vcsrepo["${archives_dir}/${repo_id}/cfg"],
    ;
    # create repository directory in /srv (the big disk)
    # This *must* include:
    #   cfg/cfg.d/
    #   cfg/lang/en/phrases/

    # now create the Puppet templated config files
    "${archives_dir}/${repo_id}/cfg/cfg.d/10_core.pl":
      ensure  => file,
      content => epp("uos_eprints_ras/10_core.pl", $template_values ),
      require => File["${archives_dir}/${repo_id}"],
      before  => Exec["create eprints tables for ${repo_id}"],
    ;
    "${archives_dir}/${repo_id}/cfg/cfg.d/database.pl":
      ensure  => file,
      content => epp("uos_eprints_ras/database.pl", $template_values ),
      require => File["${archives_dir}/${repo_id}"],
      before  => Exec["create eprints tables for ${repo_id}"],
    ;
    "${archives_dir}/${repo_id}/cfg/lang/en/phrases/puppet_phrases.xml":
      ensure  => file,
      content => epp("uos_eprints_ras/puppet_phrases.xml", $template_values ),
      require => File["${archives_dir}/${repo_id}"],
      before  => Exec["create eprints tables for ${repo_id}"],
    ;
    # now create the apache config
    "${archives_dir}/${repo_id}/ssl":
      ensure  => directory,
      require => File["${archives_dir}/${repo_id}"],
      before  => Exec["create eprints tables for ${repo_id}"],
    ;
    "${archives_dir}/${repo_id}/ssl/securevhost.conf":
      ensure  => file,
      content => epp("uos_eprints_ras/securevhost.conf", $template_values ),
      require => File["${archives_dir}/${repo_id}/ssl"],
      before  => Exec["create eprints tables for ${repo_id}"],
    ;
  }


  # create the folders that apache can write into 
  # nb. the selinux seltype is different to the config files
  file { [
      "${archives_dir}/${repo_id}/var", 
      "${archives_dir}/${repo_id}/html", 
      "${archives_dir}/${repo_id}/documents", 
      "${archives_dir}/${repo_id}/documents/disk0", 
    ]:
    ensure  => "directory", 
    seltype => "httpd_sys_rw_content_t",
    owner   => 'eprints',
    group   => 'eprints',
    notify  => Service['httpd'],
    require => File["${archives_dir}/${repo_id}"],
    before   => Exec["create eprints tables for ${repo_id}"],
  }

  # create the actual database
  mysql::db { $repo_id:
    user     => $repo_id,
    password => $mysql_password,
    host     => 'localhost',
    grant    => ['ALL'],
    before   => Exec["create eprints tables for ${repo_id}"],
  }

  # put the mysql connection info into a file 
  file { "${archives_dir}/${repo_id}/cfg/my.cnf":
    ensure  => file,
    mode    => '0644',
    owner   => 'eprints',
    group   => 'eprints',
    content => "[client]\nuser=${repo_id}\npassword=${mysql_password}",
    require => File["${archives_dir}/${repo_id}"],
    before  => Exec["create eprints tables for ${repo_id}"],
  }

  # symlink the ${archives_dir} the EPrints location in /usr/share/eprints/archives
  file { "/usr/share/eprints/archives/${repo_id}":
    ensure  => "link",
    target  => "${archives_dir}/${repo_id}",
    seltype => "httpd_sys_content_t",
    notify  => Service['httpd'],
    require => File["${archives_dir}/${repo_id}"],
    before  => Exec["create eprints tables for ${repo_id}"],
  }

  # create the SQL tables if needed
  exec { "create eprints tables for ${repo_id}":
    command => "/usr/share/eprints/bin/epadmin create_tables ${repo_id}",
    unless  => "/usr/bin/mysql --defaults-extra-file=${archives_dir}/${repo_id}/cfg/my.cnf ${repo_id} -e 'show tables' -B | /usr/bin/grep version",
    user    => 'eprints',
    group   => 'eprints',
  }

  # this all run if the config has changed
  exec { "eprints_generate_static_${repo_id}":
    command     => "/usr/share/eprints/bin/generate_static ${repo_id}",
    refreshonly => true,
    user    => 'eprints',
    group   => 'eprints',
    notify      => [ Exec["eprints_refresh_views_${repo_id}"], Exec["eprints_refresh_abstracts_${repo_id}"], Service["epindexer"] ],
  }
  exec { "eprints_refresh_views_${repo_id}":
    command     => "/usr/share/eprints/bin/epadmin refresh_views ${repo_id}",
    user    => 'eprints',
    group   => 'eprints',
    refreshonly => true,
  }
  exec { "eprints_refresh_abstracts_${repo_id}":
    command     => "/usr/share/eprints/bin/epadmin refresh_abstracts ${repo_id}",
    user    => 'eprints',
    group   => 'eprints',
    refreshonly => true,
  }


}


